-- This is a player context script for MoO2 1.50 patch project.
--
-- This tactical combat script shows a damage report of your currently active ship, showing exact shield values and
-- partial damage of subsystems and of the next weapon to be destroyed for every weapon stack.
-- There are several options which allow a different style of presentation as well as additional information to display.
-- Note that the text alignment can be a bit off sometimes due to the game not using a monospace font.
--
-- To run the script press 8 in main screen (the file should be named MAIN8.LUA)

local ALIGN_NUMBERS_RIGHT = false
local SHOW_WEAPONS_LOST_IN_SLOT_IF_ANY = false
local SHOW_COMPUTER_DAMAGE = false
local SHOW_SHIELD_GENERATOR_DAMAGE = false
local SHOW_DRIVE_DAMAGE = false
local SHOW_MARINES = false
local SHOW_ABSORBER_CHARGE_IF_PRESENT = false
local SHOW_PLASMA_WEBS_DAMAGE_IF_PRESENT = false

function main()
	local ship = get_active_combat_ship()

	local function shorten_string(str, max_length)
		local formated_string = str
		if #formated_string > max_length then
			formated_string = formated_string:sub(0, max_length - 1) .. "."
		end
		return formated_string
	end


	local function format_tech_string(str, cutoff)
		local result = str
		if str == 'anti_missile_rocket' then
			result = 'Anti-Missile Rockets'
		elseif str == 'anti_matter_bomb' then
			result = 'Anti-Matter Bomb'
		elseif str == 'anti_matter_drive' then
			result = 'Anti-Matter Drive'
		elseif str == 'anti_matter_torpedo' then
			result = 'Anti-Matter Torpedo'
		elseif str == 'anti_grav_harness' then
			result = 'Anti-Grav Harness'
		elseif str == 'bio_terminator' then
			result = 'Bio-Terminator'
		elseif str == 'class_iii_shield' then
			result = 'Class III Shield'
		elseif str == 'class_vii_shield' then
			result = 'Class VII Shield'
		elseif str == 'cyber_security_link' then
			result = 'Cyber-Security Link'
		elseif str == 'hyper_x_capacitors' then
			result = 'Hyper-X Capacitors'
		elseif str == 'multi_wave_ecm_jammer' then
			result = 'Multi-Wave Ecm Jammer'
		elseif str == 'robo_miners' then
			result = 'Robo-Miners'
		elseif str == 'sub_space_communications' then
			result = 'Sub-Space Communications'
		-- tech with variable names that differ from the equipment name
		elseif str == 'assault_shuttles' then
			result = 'Assault Shuttle'
		elseif str == 'bomber_bays' then
			result = 'Bomber'
		elseif str == 'death_spores' then
			result = 'Death Spore'
		elseif str == 'disrupter_cannon' then
			result = 'Disrupter'
		elseif str == 'fighter_bays' then
			result = 'Fighter'
		elseif str == 'heavy_fighter_bays' then
			result = 'Heavy Fighter'
		elseif str == 'shield_capacitors' then
			result = 'Shield Capacitor'
		end
		local formated_string = result:sub(1, 1):upper()
		local next_char_uppercase = false
		for i = 2, #result do
			if result:sub(i, i) == '_' then
				formated_string = formated_string .. ' '
				next_char_uppercase = true
			else
				if next_char_uppercase then
					formated_string = formated_string .. result:sub(i, i):upper()
					next_char_uppercase = false
				else
					formated_string = formated_string .. result:sub(i, i)
				end
			end
		end
		if cutoff == true then
			if formated_string == 'Multi-Wave Ecm Jammer' or formated_string == 'Structural Analyzer' then
				formated_string = shorten_string(formated_string, 17)
			else
				formated_string = shorten_string(formated_string, 16)
			end
		end
		return formated_string
	end

	local function right_whitespace_padding(shorter_string, desired_length)
		return shorter_string .. string.rep(' ', (1000*desired_length - 1250*#tostring(shorter_string))/1000)
	end

	local function left_whitespace_padding(shorter_string, desired_length)
		return string.rep(' ', (1000*desired_length - 800*#tostring(shorter_string))/1000) .. shorter_string
	end

	local function left_minipadding(shorter_string)
		if(#tostring(shorter_string)) == 1 then
			return string.rep(' ', 2) .. tostring(shorter_string)
		else
			return tostring(shorter_string)
		end
	end
			
	local function get_current_gun_hp(weapon)
		if weapon.ngood == 0 then
			return 0
		else
			return weapon.slot_HP - (weapon.slot_gun_HP * (weapon.ngood - 1))
		end
	end

	local function get_current_special_hp(special)
		if special.destroyed == 1 then
			return 0
		else
			return special.HP
		end
	end


	output = 'Damage report for ' .. ship.name .. ' (turn ' .. tostring(get_combat_info().turn + 1) .. ')\n'
	output = output .. '\n'
	local max_shield_digits = #tostring(ship.shield_max_charge)
	output = output .. string.rep(' ', 2*(2 + max_shield_digits)) .. 'F ' .. left_whitespace_padding(tostring(ship.shield_HP_arc[0]), max_shield_digits) .. string.rep(' ', 2*(2 + max_shield_digits))
	if SHOW_COMPUTER_DAMAGE == true then
		output = output .. '  Computer '
		if ship.computer_HP_base == 0 then
			output = output .. '- / -\n'
		else
			output = output .. tostring(ship.computer_HP) .. ' / ' .. tostring(ship.computer_HP_base) .. '\n'
		end
	else
		output = output .. '\n'
	end	
	output = output .. 'L ' .. left_whitespace_padding(tostring(ship.shield_HP_arc[3]), max_shield_digits) ..  string.rep(' ', 2*(1 + 2 + max_shield_digits)) .. 'R ' .. left_whitespace_padding(tostring(ship.shield_HP_arc[1]), max_shield_digits)
	if SHOW_SHIELD_GENERATOR_DAMAGE == true then
		output = output .. '  Shield      '
		if ship.shield_generator_HP_base == 0 then
			output = output .. '- / -\n'
		else
			output = output .. tostring(ship.shield_generator_HP) .. ' / ' .. tostring(ship.shield_generator_HP_base) .. '\n'
		end
	else
		output = output .. '\n'
	end
	output = output .. string.rep(' ', 2*(2 + left_whitespace_padding(max_shield_digits, max_shield_digits))) .. 'B ' .. tostring(ship.shield_HP_arc[2]) .. string.rep(' ', 2*(2 + max_shield_digits))
	if SHOW_DRIVE_DAMAGE == true then
		output = output .. '  Drive      '
		if ship.drive_HP_base == 0 then
			output = output .. '- / -\n'
		else
			output = output .. tostring(ship.drive_HP) .. ' / ' .. tostring(ship.drive_HP_base) .. '\n'
		end
	else
		output = output .. '\n'
	end	

	if SHOW_MARINES == true then
		output = output .. '\n'	
		local hull_config = get_conf("hull")
		local max_marines = hull_config[ship.size_class].marines
		hull_config = nil
		collectgarbage()
		for i, sp in pairs(ship.specials) do
			if sp.name == 'troop_pods' and sp.destroyed == 0 then
				max_marines = max_marines * 2
				break
			end
		end
		local marine_string = 'Marines'
		if ship.crew_level == 0 then
			marine_string = 'Green ' .. marine_string
		elseif ship.crew_level == 1 then
			marine_string = 'Regular ' .. marine_string
		elseif ship.crew_level == 2 then
			marine_string = 'Veteran ' .. marine_string
		elseif ship.crew_level == 3 then
			marine_string = 'Elite ' .. marine_string
		elseif ship.crew_level == 4 then
			marine_string = 'Ultra-Elite ' .. marine_string
		end
		if max_marines == 0 then
			if ALIGN_NUMBERS_RIGHT == true then
				output = output .. right_whitespace_padding(marine_string, 42) .. ' - / -\n'
			else
				output = output .. left_minipadding('-') .. ' / ' .. left_minipadding('-') .. '  ' .. marine_string .. '\n'
			end
		else
			if ALIGN_NUMBERS_RIGHT == true then
				output = output .. right_whitespace_padding(marine_string, 42) .. tostring(ship.marines) .. ' / ' .. tostring(max_marines) .. '\n'
			else
				output = output .. left_minipadding(ship.marines) .. ' / ' .. left_minipadding(max_marines) .. '  ' .. marine_string .. '\n'
			end
		end
	end

	output = output .. '\n'
	for i, w in pairs(ship.weapons) do
		local weapons_lost_in_slot_string = ''
		if SHOW_WEAPONS_LOST_IN_SLOT_IF_ANY == true then
			local weapons_lost_in_slot = w.slot_HP_base / w.slot_gun_HP - w.ngood
			if weapons_lost_in_slot > 0 then
				weapons_lost_in_slot_string = ' lost ' .. tostring(weapons_lost_in_slot)
			end
		end
		if w.slot_gun_HP == 0 then
			if ALIGN_NUMBERS_RIGHT == true then
				output = output .. right_whitespace_padding(format_tech_string(w.type), 42) .. tostring('-') .. ' / ' .. tostring('-') .. '\n'			
			else
				output = output .. left_minipadding('-') .. ' / ' .. left_minipadding('-') .. '  ' .. format_tech_string(w.type) .. '\n'
			end
		else
			if ALIGN_NUMBERS_RIGHT == true then
				output = output .. right_whitespace_padding(format_tech_string(w.type), 42) .. tostring(get_current_gun_hp(w)) .. ' / ' .. tostring(w.slot_gun_HP) .. weapons_lost_in_slot_string .. '\n'			
			else
				output = output .. left_minipadding(get_current_gun_hp(w)) .. ' / ' .. left_minipadding(w.slot_gun_HP) .. weapons_lost_in_slot_string .. '  ' .. tostring(format_tech_string(w.type)) .. '\n'
			end
		end
	end

	local has_absorber = false
	output = output .. '\n'	
	for i, sp in pairs(ship.specials) do
		if sp.HP_base == 0 then
			if ALIGN_NUMBERS_RIGHT == true then
				output = output .. right_whitespace_padding(format_tech_string(sp.name), 42) .. tostring('-') .. ' / ' .. tostring('-') .. '\n'			
			else
				output = output .. left_minipadding('-') .. ' / ' .. left_minipadding('-') .. '  ' .. format_tech_string(sp.name) .. '\n'
			end
		else
			if ALIGN_NUMBERS_RIGHT == true then
				output = output.. right_whitespace_padding(format_tech_string(sp.name), 42) .. tostring(get_current_special_hp(sp)) .. ' / ' .. tostring(sp.HP_base) .. '\n'
			else
				output = output .. left_minipadding(get_current_special_hp(sp)) .. ' / ' .. left_minipadding(sp.HP_base) .. '  ' .. format_tech_string(sp.name) .. '\n'
			end
		end
		if sp.name == 'energy_absorber' then
			has_absorber = true
		end
	end
	if has_absorber == true and ship.absorber_charge > 0 and SHOW_ABSORBER_CHARGE_IF_PRESENT == true then
		output = output .. '\n'	
		output = output .. 'Damage stored in absorber ' .. tostring(ship.absorber_charge)
	end
	if ship.plasma_webs_damage >0 and SHOW_ABSORBER_CHARGE_IF_PRESENT == true then
		output = output .. '\n'	
		output = output .. 'Damage accumulated by plasma webs ' .. tostring(ship.plasma_webs_damage)
	end
	msgbox(output)
end

if type(get_active_combat_ship) ~= "function" then
	msgbox('report for tactical combat')
else
	main()
end